
import java.awt.*;
import Simulation;
import MDDemo;
import Atom;

public class ControlsPanel extends Panel {
    public TextField applTempField;
    public TextField densField;
    public Scrollbar tempSlider;
    public Scrollbar densSlider;
    private Checkbox scalingCheckbox;
    private Simulation s;
    private Font f1 = new Font("TimesRoman",Font.BOLD,14);
    private MDInterface parent;
    private double density;
    private Color tfbg = Color.yellow;
    private Font tffont = new Font("TimesRoman",Font.BOLD,12);
    private Color labelColor = new Color(150,150,200);

    public ControlsPanel(Simulation s,MDInterface parent) {
	this.s = s;
	this.parent = parent;
	GridBagLayout gridBag = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	setLayout(gridBag);

	c.fill = GridBagConstraints.HORIZONTAL;
	c.weighty = 100.0;
	c.weightx = 100.0;
	c.gridx = GridBagConstraints.RELATIVE;
	c.gridy = GridBagConstraints.RELATIVE;
	c.gridwidth = 1;
	c.gridheight = 1;
	
	scalingCheckbox = new Checkbox("Velocity Scaling");
	scalingCheckbox.setState(true);
	scalingCheckbox.setFont(f1);
	scalingCheckbox.setBackground(labelColor);
	c.gridwidth = GridBagConstraints.REMAINDER;
	gridBag.setConstraints(scalingCheckbox,c);
	add(scalingCheckbox);
	
	Label label1 = new Label("Applied Temp:",Label.LEFT);
	label1.setFont(f1);
	label1.setBackground(labelColor);
	c.gridwidth = 1;
	gridBag.setConstraints(label1,c);
	add(label1);
	
	applTempField = new TextField("150",6);
	s.setApplTemp(150.0);
	c.gridwidth = GridBagConstraints.REMAINDER;
	gridBag.setConstraints(applTempField,c);
	applTempField.setBackground(tfbg);
	applTempField.setFont(tffont);
	add(applTempField);
	applTempField.setEditable(false);
	
	tempSlider = new Scrollbar(Scrollbar.HORIZONTAL,150,20,10,1500);
	tempSlider.setPageIncrement(50);
	tempSlider.setLineIncrement(1);
	gridBag.setConstraints(tempSlider,c);
	add(tempSlider);
	
	Label label2 = new Label("Density:",Label.LEFT);
	label2.setFont(f1);
	label2.setBackground(labelColor);
	c.gridwidth = 1;
	gridBag.setConstraints(label2,c);
	add(label2);
	
	densField = new TextField("0.069",6);
	c.gridwidth = GridBagConstraints.REMAINDER;
	gridBag.setConstraints(densField,c);
	densField.setBackground(tfbg);
	densField.setFont(tffont);
	add(densField);
	densField.setEditable(false);
	
	densSlider = new Scrollbar(Scrollbar.HORIZONTAL,69,10,10,150);
	densSlider.setPageIncrement(20);
	densSlider.setLineIncrement(1);
	c.fill = GridBagConstraints.HORIZONTAL;
	c.gridwidth = GridBagConstraints.REMAINDER;
	gridBag.setConstraints(densSlider,c);
	add(densSlider);
	updateDensity();
    }

    public boolean handleEvent(Event e) {
	if(e.id == Event.SCROLL_ABSOLUTE ||
	   e.id == Event.SCROLL_LINE_DOWN ||
	   e.id == Event.SCROLL_LINE_UP ||
	   e.id == Event.SCROLL_PAGE_DOWN ||
	   e.id == Event.SCROLL_PAGE_UP) {
	    int n;
	    if (e.target == (Object)densSlider) {
		updateDensity();
		String str = String.valueOf(density);
		if(str.length() > 6) n = 6;
		else n = str.length();
		densField.setText(str.substring(0,n));
		return true;
	    }
	    if (e.target == (Object)tempSlider) {
		applTempField.setText(String.valueOf(tempSlider.getValue()));
		s.setApplTemp( tempSlider.getValue() );
		return true;
	    }
	}
	return super.handleEvent(e);
    }

    public boolean action(Event e, Object arg) {
	if(e.target.equals(scalingCheckbox)) {
	    s.setScaling(scalingCheckbox.getState());
	    return true;
	}
	return true;
    }
  
    private void updateDensity() {
	int d;
	
	d = densSlider.getValue();
	density = (double)d * 0.001;
    }
    
    public double getDensity() {
	return density;
    }
}
