
import java.awt.*;
import SimulationRegion;
import CardPanel;

public class MDInterface extends Panel {
  private Label edgeLabel;
  SimulationRegion regionCanvas;
  MDDemo controller;
  Button startButton;
  Button stopButton;
  Button resetButton;
  Simulation s;
  private ControlsPanel cp;
  private boolean threadRunning = false;
  private boolean threadSuspended = false;
  private Font edgeFont = new Font("TimesRoman",Font.PLAIN,14);
  private Font fbig = new Font("TimesRoman",Font.BOLD,24);
  private CardPanel cards;
  private Color bgcolor = new Color(170,170,170);
  private Color fgcolor = new Color(20,20,20);

  public MDInterface(MDDemo mycontroller, Simulation s) {
    this.s = s;
    GridBagLayout gridBag = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();
    setLayout(gridBag);
    setBackground(bgcolor);
    setForeground(fgcolor);

    //Save arguments in instance variables
    controller = mycontroller;
    this.s = s;

    //Default constraints
    c.fill = GridBagConstraints.HORIZONTAL;
    c.weighty = 100.0;
    c.weightx = 100.0;
    c.gridx = GridBagConstraints.RELATIVE;
    c.gridy = GridBagConstraints.RELATIVE;

    Label label1 = new Label("Java Molecular Dynamics",Label.CENTER);
    label1.setFont(fbig);
    c.gridwidth = GridBagConstraints.REMAINDER;
    c.gridheight = 1;
    gridBag.setConstraints(label1,c);
    add(label1);
    
    regionCanvas = new SimulationRegion(this,s);
    c.fill = GridBagConstraints.BOTH;
    c.gridwidth = 1;
    c.gridheight = 1;
    gridBag.setConstraints(regionCanvas, c);
    add(regionCanvas);

    cp = new ControlsPanel(s,this);
    c.gridwidth = GridBagConstraints.REMAINDER;
    gridBag.setConstraints(cp,c);
    add(cp);

    String str = String.valueOf(s.regionEdge);
    int n;
    if(str.length() > 6) n = 6;
    else n = str.length();
    edgeLabel = new Label(str.substring(0,n) + " Angstroms",Label.CENTER);
    edgeLabel.setFont(edgeFont);
    c.fill = GridBagConstraints.HORIZONTAL;
    c.gridwidth = 1;
    c.weightx = 100;
    gridBag.setConstraints(edgeLabel,c);
    add(edgeLabel);

    Label dummyLabel = new Label("",Label.LEFT);
    c.gridwidth = GridBagConstraints.REMAINDER;
    gridBag.setConstraints(dummyLabel,c);
    add(dummyLabel);

    Panel p = new Panel();
    p.setLayout(new FlowLayout());

    startButton = new Button("Start");
    p.add(startButton);

    stopButton = new Button("Stop");
    p.add(stopButton);

    resetButton = new Button("Reset");
    p.add(resetButton);

    gridBag.setConstraints(p,c);
    add(p);

    cards = new CardPanel(s);
    gridBag.setConstraints(cards,c);
    add(cards);
  }

    public void update() {
	cards.update();
	regionCanvas.repaint();
    }

  public boolean action(Event e, Object arg) {
    if(arg.equals("Stop")) {
      if(threadRunning) {
	s.suspend();
	threadSuspended = true;
	threadRunning = false;
	return true;
      }
    } else if(arg.equals("Start")) {
      if(!threadRunning && !threadSuspended) {
	  threadRunning = true;
	  threadSuspended = false;
	s.start();
	return true;
      }
      if(threadSuspended) {
	s.resume();
	threadSuspended = false;
	threadRunning = true;
	return true;
      }
    } else if(arg.equals("Reset")) {
      if(threadRunning) {
	s.suspend();
	threadSuspended = true;
	threadRunning = false;
      }
      s.resetSim();
    }
    return true;
  }

    public void reset() {
	regionCanvas.repaint();
    }

    public double getDensity() {
	return cp.getDensity();
    }

    public void setEdgeLabel(double l) {
	String str;
	int n;

	str = String.valueOf(l);
	if(str.length() > 6) n = 6;
	else n = str.length();
	edgeLabel.setText(str.substring(0,n) + " Angstroms");
    }
}
