
import java.awt.*;
import java.util.*;
import Simulation;

public class RDFPanel extends Panel {
    private LinePlot plot;
    private int count = 0;
    private Simulation s;
    private Label title;
    private int NAVERAGE = 100;
    private int SIZE = 500;
    private double[] data;

    public RDFPanel(Simulation s) {
	plot = new LinePlot();
	this.s = s;
	data = new double[SIZE];
	
	setLayout(new BorderLayout());
	setFont(new Font("TimesRoman",Font.PLAIN,14));
	title = new Label("Radial Distribution Function",Label.CENTER);
	add("North",title);
	add("South",new Label("r",Label.CENTER));
	add("West",new Label("g(r)",Label.RIGHT));
	add("Center",plot);
    }

    public void update() {
	Enumeration e1,e2;
	Atom a1,a2;
	double rij2,rij;
	double rijx,rijy;
	double ri2,ri12,ri6;
	double cutoff = 10.0;
	double cutoff2 = cutoff * cutoff;
	int n;
	double deltaR = cutoff / SIZE;

	count++;
	if(count == 1) {
	    for(int i = 0; i < SIZE; i++) {
		data[i] = 0.0;
	    }
	}
	if(count == NAVERAGE + 1) {
	    display(deltaR);
	    count = 1;
	}
	setTitle((double)count / NAVERAGE);
    
	e1 = s.atoms.elements();
	e2 = s.atoms.elements();
	
	while(e1.hasMoreElements()) {
	    a1 = (Atom)e1.nextElement();
	    e2 = s.atoms.elements();
	    while(e2.hasMoreElements()) {
		a2 = (Atom)e2.nextElement();
		if(a2 != a1) {
		    rijx = a1.x - a2.x;
		    if(rijx > s.hregionEdge) rijx = rijx - s.regionEdge;
		    if(rijx < -s.hregionEdge) rijx = rijx + s.regionEdge;
		    rijy = a1.y - a2.y;
		    if(rijy > s.hregionEdge) rijy = rijy - s.regionEdge;
		    if(rijy < -s.hregionEdge) rijy = rijy + s.regionEdge;
		    rij2 = (rijx * rijx) + (rijy * rijy);
		    if(rij2 < cutoff2) {
			n = (int)Math.round(Math.sqrt(rij2) / deltaR);
			if(n >= SIZE) n = SIZE -1;
			data[n] = data[n] + 1.0;
		    }
		}
	    }
	}
    }

    private void display(double deltaR) {
	double factor,r;
	
	factor = (s.regionEdge * s.regionEdge) / 
	    (2.0 * Math.PI * s.N_ATOMS * s.N_ATOMS * NAVERAGE * deltaR);
	for(int i = 0; i < SIZE ; i++) {
	    r = ((double)i + 0.5) * deltaR;
	    data[i] = (data[i] * factor) / r;
	}
	plot.plotData(data,deltaR,0.0);
    }

    private void setTitle(double n) {
	String s = double2string(n * 100.0);
	title.setText("Radial Distribution Function  (Computing... "+s+"% )");
    }

    private String double2string(double n) {
	return( new Double( Math.rint(n) ).toString() );
    }

}
